/*
 * A small number of WAV-format related functions.  These were mainly
 * grabbed from wavnorm 0.4.
 *
 * wavnorm was written by "Duncan", duncan@zog.net.au
 *
 * Original information came from
 *   http://www-ccrma.stanford.edu/CCRMA/Courses/422/projects/WaveFormat/
 *
 */

#include <math.h>
#include <sys/types.h>
#include <string.h>
#include "wavutils.h"

/* ======================================================================== */

void create_wav_header(WAVE_HEADER *wheader,
   int nChannels,int nSamplesPerSec,int wBitsPerSample) {
/*
 * Purpose: initialise a wav-file header with sensible default.
 *
 */
   /* first 4 bytes should be RIFF */
   strncpy(wheader->RiffID,"RIFF",4);

   /* next 4 bytes should be WAVE */
   strncpy(wheader->WaveID,"WAVE",4);

   /* next 4 bytes should be "fmt " */
   strncpy(wheader->FmtID,"fmt ",4);

   wheader->FmtSize = 16;
   wheader->wFormatTag = 1;

   wheader->nChannels = nChannels;
   wheader->nSamplesPerSec = nSamplesPerSec;
   wheader->nAvgBytesPerSec = nSamplesPerSec * nChannels * wBitsPerSample/8;
   wheader->wBitsPerSample = wBitsPerSample;
   wheader->nBlockAlign = nChannels * wBitsPerSample/8;
   wheader->nDataBytes = 0;

   strncpy(wheader->DataID,"data",4);
}
/* ======================================================================== */

short get_peak_value(short *start, short *end) {
/*
 * Purpose: returns the peak value between the two pointers supplied.
 *
 */
short the_max = 0;
short *current = start;

  while (current < end) {
    if (abs(*current) > the_max)
      the_max = abs(*current);
    current++;
  }
  return the_max;
}
/* ======================================================================== */

void get_stereo_peak_values(short *start, short *end, short *l, short *r) {
/*
 * Purpose: returns the peak value of two interleved channels between the
 *          two pointers supplied.
 *
 * Notes: it is assumed that both channels have the same number of samples.
 *
 */
short the_max_l = 0;
short the_max_r = 0;
short *current = start;

  while (current < end) {
    if (abs(*current) > the_max_l)
      the_max_l = abs(*current);
    if (abs(*(current+1)) > the_max_r)
      the_max_r = abs(*(current+1));
    current += 2;
  }
  *l = the_max_l;
  *r = the_max_r;
}
/* ======================================================================== */

void get_stereo_ave_values(short *start, short *end, short *l, short *r) {
/*
 * Purpose: returns the average value of two interleved channels between the
 *          two pointers supplied.
 *
 * Notes: it is assumed that both channels have the same number of samples.
 *
 */
float ave_l = 0.0;
float ave_r = 0.0;
short *current = start;
int n = 0;

  while (current < end) {
    ave_l += abs(*current);
    ave_r += abs(*(current+1));
    current += 2;
    n++;
  }
  *l = (short)rint(ave_l/n);
  *r = (short)rint(ave_r/n);
}
/* ======================================================================== */
